/***************************************************************************
                          fractalalgorithminterface.cpp  -  description
                             -------------------
    begin                : Son Mai 11 2003
    copyright            : (C) 2003 by Thomas Dreibholz
    email                : dreibh@exp-math.uni-essen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fractalalgorithminterface.h"
#include "uintconfigentry.h"


QPtrList<FractalAlgorithmInterface> FractalAlgorithmInterface::AlgorithmList;

   
FractalAlgorithmInterface::FractalAlgorithmInterface()
{
   AlgorithmList.append(this);

   // When using this mechanism to allow configuration of own variables
   // in derived classes you need to make sure the string could be used in
   // well-formed xml as a tag (i.e. no whitespaces)
   ConfigEntries.append(new UIntConfigEntry(&MaxIterations, "MaxIterations"));
}

FractalAlgorithmInterface::~FractalAlgorithmInterface()
{
   AlgorithmList.remove(this);
}

std::complex<double> FractalAlgorithmInterface::defaultC1()
{
   return(std::complex<double>(0.0, 0.0));
}

std::complex<double> FractalAlgorithmInterface::defaultC2()
{
   return(std::complex<double>(0.0, 0.0));
}

int FractalAlgorithmInterface::defaultMaxIterations()
{
   return 100;
}

void FractalAlgorithmInterface::configure(unsigned int         width,
                                          unsigned int         height,
                                          std::complex<double> c1,
                                          std::complex<double> c2,
                                          unsigned int         maxIterations)
{                                          
   Width         = width;
   Height        = height;
   MaxIterations = maxIterations;
   C1            = c1;
   C2            = c2;
   StepX         = (c2.real() - c1.real()) / Width;
   StepY         = (c2.imag() - c1.imag()) / Height;
   printf("STEPS: %f %f\n",StepX, StepY);
}

FractalAlgorithmInterface* FractalAlgorithmInterface::getAlgorithm(const unsigned int index)
{
   return(AlgorithmList.at(index));  
}


void FractalAlgorithmInterface::changeSize(int X, int Y)
{
	Width = X;
	Height = Y;
	StepX         = (C2.real() - C1.real()) / Width;
	StepY         = (C2.imag() - C1.imag()) / Height;
	printf("STEPS: %f %f\n",StepX, StepY);
}
