/***************************************************************************
                          fractalgenerator.cpp  -  description
                             -------------------
    begin                : Sam Mai 10 10:39:46 CEST 2003
    copyright            : (C) 2003 by Thomas Dreibholz
    email                : dreibh@exp-math.uni-essen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qpainter.h>
#include <qinputdialog.h>
#include <qmessagebox.h>
#include <qdom.h>

// include files for KDE
#include <kprinter.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>

// application specific includes
#include "fractalgenerator.h"
#include "fractalgenerator.moc"
#include "fractalgeneratorview.h"
#include "fractalgeneratordoc.h"
#include "fractalalgorithminterface.h"
#include "colorschemeinterface.h"
#include "optionsdialog.h"
#include <iostream>

#define ID_STATUS_MSG 1

FractalGeneratorApp::FractalGeneratorApp(QWidget* , const char* name):KMainWindow(0, name)
{
  config=kapp->config();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initStatusBar();
  initActions();
  initDocument();
  initView();

  readOptions();

  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
  fileSave->setEnabled(false);
  fileSaveAs->setEnabled(true);
  filePrint->setEnabled(true);
  settingsZoomBack->setEnabled(false);
  
}

FractalGeneratorApp::~FractalGeneratorApp()
{

}

void FractalGeneratorApp::initActions()
{
  fileNewWindow = new KAction(i18n("New &Window"), 0, 0, this, SLOT(slotFileNewWindow()), actionCollection(),"file_new_window");
  //fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
  fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
  fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
  fileExportPicture =new KAction(i18n("&Export Picture"), 0, 0, this, SLOT(slotFileExportPicture()), actionCollection(),"export_picture");
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
  settingsConfigureAlgorithm = new KAction(i18n("Configure &Algorithm"), 0, 0, this, SLOT(slotSettingsConfigureAlgorithm()), actionCollection(),"configure_algorithm");
  settingsSetPictureSize = new KAction(i18n("Change Size"), 0, 0, this, SLOT(slotSettingsSetPictureSize()), actionCollection(),"set_picture_size");
  settingsResetZoom = new KAction(i18n("Reset Zoom"), 0, 0, this, SLOT(slotSettingsResetZoom()), actionCollection(),"reset_zoom");
  settingsZoomBack = new KAction(i18n("Zoom Back"), 0, 0, this, SLOT(slotSettingsZoomBack()), actionCollection(),"zoom_back");
  editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
  editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
  editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());
  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());

  fileNewWindow->setStatusText(i18n("Opens a new application window"));
  fileExportPicture->setStatusText(i18n("Exports the Picture to a file"));
  //fileNew->setStatusText(i18n("Creates a new document"));
  fileOpen->setStatusText(i18n("Opens an existing document"));
  fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  fileSave->setStatusText(i18n("Saves the actual document"));
  fileSaveAs->setStatusText(i18n("Saves the actual document as..."));
  fileClose->setStatusText(i18n("Closes the actual document"));
  filePrint ->setStatusText(i18n("Prints out the actual document"));
  fileQuit->setStatusText(i18n("Quits the application"));
  settingsConfigureAlgorithm->setStatusText(i18n("Configure Parameters for the algorithm"));
  settingsSetPictureSize->setStatusText(i18n("Configure the size of the fractal"));
  settingsResetZoom->setStatusText(i18n("Set Zoom to standard"));
  settingsZoomBack->setStatusText(i18n("Return to previous zoom settings"));
  editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
  editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
  editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));
  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));
     
  // use the absolute path to your fractalgeneratorui.rc file for testing purpose in createGUI();
  createGUI("fractalgeneratorui.rc");

  selectAlgorithm   = new KSelectAction(i18n("Algorithm"));
  connect(selectAlgorithm, SIGNAL(activated(int)), SLOT(slotAlgorithmChanged(int)));
  unsigned int i = 0;
  FractalAlgorithmInterface* algorithm;
  QStringList                algorithmList;
  while((algorithm = FractalAlgorithmInterface::getAlgorithm(i++))) {
     algorithmList.append(algorithm->getName());
  }
  selectAlgorithm->setItems(algorithmList);
  selectAlgorithm->plug(toolBar());
  
  selectColorScheme = new KSelectAction(i18n("Color Scheme"));
  connect(selectColorScheme, SIGNAL(activated(int)), SLOT(slotColorSchemeChanged(int)));
  i = 0;
  ColorSchemeInterface* colorScheme;
  QStringList           colorSchemeList;
  while((colorScheme = ColorSchemeInterface::getColorScheme(i++))) {
     colorSchemeList.append(colorScheme->getName());
  }
  selectColorScheme->setItems(colorSchemeList);
  selectColorScheme->plug(toolBar());

  
}


void FractalGeneratorApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
}

void FractalGeneratorApp::initDocument()
{
  doc = new FractalGeneratorDoc(this);
  doc->newDocument();
}

void FractalGeneratorApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new FractalGeneratorView(this);
  doc->addView(view);
  setCentralWidget(view);
  setCaption(doc->URL().fileName(),false);

}

void FractalGeneratorApp::openDocumentFile(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));

  doc->openDocument( url);
  fileOpenRecent->addURL( url );
  slotStatusMsg(i18n("Ready."));
}


FractalGeneratorDoc *FractalGeneratorApp::getDocument() const
{
  return doc;
}

void FractalGeneratorApp::saveOptions()
{
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", viewToolBar->isChecked());
  config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
  config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
  fileOpenRecent->saveEntries(config,"Recent Files");
}


void FractalGeneratorApp::readOptions()
{

  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewToolBar->setChecked(bViewToolbar);
  slotViewToolBar();

  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewStatusBar->setChecked(bViewStatusbar);
  slotViewStatusBar();


  // bar position settings
  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);
  toolBar("mainToolBar")->setBarPos(toolBarPos);

  // initialize the recent file list
  fileOpenRecent->loadEntries(config,"Recent Files");

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
}

void FractalGeneratorApp::saveProperties(KConfig *_cfg)
{
  if(doc->URL().fileName()!=i18n("Untitled") && !doc->isModified())
  {
    // saving to tempfile not necessary

  }
  else
  {
    KURL url=doc->URL();
    _cfg->writeEntry("filename", url.url());
    _cfg->writeEntry("modified", doc->isModified());
    QString tempname = kapp->tempSaveName(url.url());
    QString tempurl= KURL::encode_string(tempname);
    KURL _url(tempurl);
    doc->saveDocument(_url);
  }
}


void FractalGeneratorApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filename", "");
  KURL url(filename);
  bool modified = _cfg->readBoolEntry("modified", false);
  if(modified)
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
    KURL _url(tempname);

    if(canRecover)
    {
      doc->openDocument(_url);
      doc->setModified();
      setCaption(_url.fileName(),true);
      QFile::remove(tempname);
    }
  }
  else
  {
    if(!filename.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(),false);
    }
  }
}

bool FractalGeneratorApp::queryClose()
{
  return doc->saveModified();
}

bool FractalGeneratorApp::queryExit()
{
  saveOptions();
  return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void FractalGeneratorApp::slotFileNewWindow()
{
  slotStatusMsg(i18n("Opening a new application window..."));

  FractalGeneratorApp *new_window= new FractalGeneratorApp();
  new_window->show();

  slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {
    doc->newDocument();
    setCaption(doc->URL().fileName(), false);
  }

  slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));

  if(!doc->saveModified())
  {
     // here saving wasn't successful

  }
  else
  {
    KURL url=KFileDialog::getOpenURL(QString::null,
        i18n("*.fsf|Fractal Save Files"), this, i18n("Open File..."));
    if(!url.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(), false);
      fileOpenRecent->addURL( url );
      fileSave->setEnabled(true);
    }
  }
  settingsZoomBack->setEnabled(view->isZoomBackPossible());
  slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));

  if(!doc->saveModified())
  {
     // here saving wasn't successful
  }
  else
  {
    doc->openDocument(url);
    setCaption(url.fileName(), false);
  }

  settingsZoomBack->setEnabled(view->isZoomBackPossible());
  slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));

  doc->saveDocument(doc->URL());

  slotStatusMsg(i18n("Ready."));
}


void FractalGeneratorApp::slotAlgorithmChanged(int index)
{
    slotStatusMsg(i18n("Changing Algorithm..."));

    view->changeAlgorithm(index);
    view->configChanged();
    settingsZoomBack->setEnabled(view->isZoomBackPossible());

    slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotColorSchemeChanged(int index)
{

    slotStatusMsg(i18n("Changing ColorScheme..."));

    view->changeColorScheme(index);
    view->configChanged();
    settingsZoomBack->setEnabled(view->isZoomBackPossible());

    slotStatusMsg(i18n("Ready."));
}


void FractalGeneratorApp::slotFileSaveAs()
{
  slotStatusMsg(i18n("Saving file with a new filename..."));

  KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*.fsf|Fractal Save Files"), this, i18n("Save as..."));
  if(!url.isEmpty())
  {
    doc->saveDocument(url);
    fileOpenRecent->addURL(url);
    setCaption(url.fileName(),doc->isModified());
    fileSave->setEnabled(true);
  }

  slotStatusMsg(i18n("Ready."));
}


void FractalGeneratorApp::slotFileExportPicture()
{
  slotStatusMsg(i18n("Exporting Picture..."));

  KURL url=KFileDialog::getSaveURL(QDir::currentDirPath(),
        i18n("*.png|*.png"), this, i18n("Save Picture as..."));
  if(!url.isEmpty())
  {
        QString path = url.prettyURL().mid(5);
        if(path.right(4).lower() != i18n(".png"))
            path += i18n(".png");
        view->getDisplay()->savePicture(path, "PNG");
  }

  slotStatusMsg(i18n("Ready."));
}


void FractalGeneratorApp::slotSettingsConfigureAlgorithm()
{
	slotStatusMsg(i18n("Changing Options..."));

    OptionsDialog Dialog(view, "Change/View Options", view->getAlgorithm()->getConfigEntries());
    
    Dialog.exec();
    view->configChanged();

    slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));

  close();

  slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotFilePrint()
{
   slotStatusMsg(i18n("Printing..."));

   KPrinter printer;
   printer.setColorMode(KPrinter::Color);
   printer.setOrientation(KPrinter::Landscape);
   printer.setOutputFileName("/tmp/fractal.ps");
   printer.setOutputToFile(true);
   if(printer.setup(this)) {
      view->print(&printer);
   }

   slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotFileQuit()
{
  puts("Slot-QUIT!");

  slotStatusMsg(i18n("Exiting..."));
puts("SQ1");    
  saveOptions();
puts("SQ2");  
  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  /*
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
puts("SQ3");    
      if(!w->close()) {
puts("SQ-STOP!");            
         break;
      }
    }
  }  // ????????????????????????
  */
  puts("Exit!");
  exit(0);
}

void FractalGeneratorApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));

  slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));

  slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));

  slotStatusMsg(i18n("Ready."));
}


void FractalGeneratorApp::slotSettingsSetPictureSize()
{
  slotStatusMsg(i18n("Modifying size of Picture..."));
  QString CurrentSize;

  CurrentSize += QString().setNum(view->getSizeX());
  CurrentSize += i18n("*");
  CurrentSize += QString().setNum(view->getSizeY());
  
  bool ok;
  QString text = QInputDialog::getText(
            "Fractalgenerator", "Please enter new size in the format x*y:", QLineEdit::Normal,
            CurrentSize, &ok, this );
  if(ok || !text.isEmpty())
  {
    unsigned int NewX = text.section("*",0,0).toUInt();
    unsigned int NewY = text.section("*",1,1).toUInt();
    
    if((0 < NewX) && (0 < NewY))
    {
      view->changeSize(NewX, NewY);
      view->configChanged();
    }
    else
    {
      QMessageBox::information( this, "Fractalgenerator",
        "Change to " 
        + QString().setNum(NewX)
        + "*"
        + QString().setNum(NewY)
        + " failed!");
       }
       settingsZoomBack->setEnabled(view->isZoomBackPossible());
  }

               
  slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }

  slotStatusMsg(i18n("Ready."));
}

void FractalGeneratorApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}


void FractalGeneratorApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}


void FractalGeneratorApp::slotSettingsResetZoom()
{
    slotStatusMsg(i18n("Reset Zoom..."));

    view->resetZoom();
    view->configChanged();
    settingsZoomBack->setEnabled(view->isZoomBackPossible());

    slotStatusMsg(i18n("Ready."));
}


void FractalGeneratorApp::slotSettingsZoomBack()
{
    slotStatusMsg(i18n("Zoom Back..."));

    view->zoomBack();
    view->configChanged();

    settingsZoomBack->setEnabled(view->isZoomBackPossible());

    slotStatusMsg(i18n("Ready."));
}


void FractalGeneratorApp::updateZoomBackPossible()
{
    settingsZoomBack->setEnabled(view->isZoomBackPossible());
}


