/***************************************************************************
                          fractalgeneratordoc.cpp  -  description
                             -------------------
    begin                : Sam Mai 10 10:39:46 CEST 2003
    copyright            : (C) 2003 by Thomas Dreibholz
    email                : dreibh@exp-math.uni-essen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qwidget.h>
#include <qfile.h>
#include <qdatastream.h>
#include <qmessagebox.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>

// application specific includes
#include "fractalgeneratordoc.h"
#include "fractalgenerator.h"
#include "fractalgeneratordoc.moc"
#include "fractalgenerator.h"
#include "fractalgeneratorview.h"
#include <iostream>

QList<FractalGeneratorView> *FractalGeneratorDoc::pViewList = 0L;

FractalGeneratorDoc::FractalGeneratorDoc(QWidget *parent, const char *name) : QObject(parent, name)
{
  if(!pViewList)
  {
    pViewList = new QList<FractalGeneratorView>();
  }

  pViewList->setAutoDelete(true);

  app = static_cast<FractalGeneratorApp*>(parent);
}

FractalGeneratorDoc::~FractalGeneratorDoc()
{
}

void FractalGeneratorDoc::addView(FractalGeneratorView *view)
{
  pViewList->append(view);
}

void FractalGeneratorDoc::removeView(FractalGeneratorView *view)
{
  pViewList->remove(view);
}
void FractalGeneratorDoc::setURL(const KURL &url)
{
  doc_url=url;
}

const KURL& FractalGeneratorDoc::URL() const
{
  return doc_url;
}

void FractalGeneratorDoc::slotUpdateAllViews(FractalGeneratorView *sender)
{
  FractalGeneratorView *w;
  if(pViewList)
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
      if(w!=sender)
        w->repaint();
    }
  }

}

bool FractalGeneratorDoc::saveModified()
{
  bool completed=true;

  if(modified)
  {
    FractalGeneratorApp *win=(FractalGeneratorApp *) parent();
    int want_save = KMessageBox::warningYesNoCancel(win,
                                         i18n("The current file has been modified.\n"
                                              "Do you want to save it?"),
                                         i18n("Warning"));
    switch(want_save)
    {
      case KMessageBox::Yes:
           if (doc_url.fileName() == i18n("Untitled"))
           {
             win->slotFileSaveAs();
           }
           else
           {
             saveDocument(URL());
       	   };

       	   deleteContents();
           completed=true;
           break;

      case KMessageBox::No:
           setModified(false);
           deleteContents();
           completed=true;
           break;

      case KMessageBox::Cancel:
           completed=false;
           break;

      default:
           completed=false;
           break;
    }
  }

  return completed;
}

void FractalGeneratorDoc::closeDocument()
{
  deleteContents();
}

bool FractalGeneratorDoc::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
  modified=false;
  doc_url.setFileName(i18n("Untitled"));

  return true;
}

bool FractalGeneratorDoc::openDocument(const KURL& url, const char *format /*=0*/)
{
  
  QString tmpfile;
  KIO::NetAccess::download( url, tmpfile );
  
  QDomDocument doc( "XMLFractalSave" );
  QFile file( tmpfile);//url.prettyURL().mid(5) );
  if ( !file.open( IO_ReadOnly ) )
    return false;
      
  QString Error;
  int Line, Column;
  if ( !doc.setContent( &file , false, &Error, &Line, &Column) ) {
    file.close();
    QMessageBox::information(0, "Fractalgenerator", Error + " in Line:" + QString().setNum(Line)
       + " and Column: " + QString().setNum(Column));
    return false;
  }
  file.close();
  KIO::NetAccess::removeTempFile( tmpfile );

  QDomElement algorithm = doc.elementsByTagName("AlgorithmName").item(0).toElement();
  QString AlgorithmName = algorithm.firstChild().toText().data();

  FractalGeneratorView *view = pViewList->getFirst();
        
  unsigned int i = 0;                 
  FractalAlgorithmInterface* algorithminterface;
  while((algorithminterface = FractalAlgorithmInterface::getAlgorithm(i)))
  {
     if(QString(algorithminterface->getSaveIdentifier())==AlgorithmName)
     {
		 view->changeAlgorithm(i);
		 app->selectAlgorithm->setCurrentItem(i);
		 break;
	 }
	 i++;
  }

  QDomElement colorscheme = doc.elementsByTagName("ColorSchemeName").item(0).toElement();
  QString ColorSchemeName = colorscheme.firstChild().toText().data();

  i = 0;
  ColorSchemeInterface* colorschemeinterface;
  while((colorschemeinterface = ColorSchemeInterface::getColorScheme(i)))
  {
     if(QString(colorschemeinterface->getSaveIdentifier())==ColorSchemeName)
     {
		 view->changeColorScheme(i);
		 app->selectColorScheme->setCurrentItem(i);
		 break;
	 }
	 i++;
  }
  /*
  QString resolution = doc.elementsByTagName("Resolution").item(0).firstChild().toText().data();
    
  unsigned int NewX = resolution.section("*",0,0).toUInt();
  unsigned int NewY = resolution.section("*",1,1).toUInt();
  view->changeSize(NewX, NewY);
  */

  
  double c1real = doc.elementsByTagName("C1Real").item(0).firstChild().toText().data().toDouble();
  double c1imag = doc.elementsByTagName("C1Imag").item(0).firstChild().toText().data().toDouble();
  double c2real = doc.elementsByTagName("C2Real").item(0).firstChild().toText().data().toDouble();
  double c2imag = doc.elementsByTagName("C2Imag").item(0).firstChild().toText().data().toDouble();
  std::complex<double> C1(c1real, c1imag);
  std::complex<double> C2(c2real, c2imag);
  view->changeC1C2(C1, C2);

  QDomElement useroptions = doc.elementsByTagName("Useroptions").item(0).toElement();
  QDomNode child = useroptions.firstChild();
  while(!child.isNull())
  {
	  QString Name = child.nodeName();
	  QString Value = child.firstChild().toText().data();
	  QPtrListIterator<ConfigEntry> it(*(view->getAlgorithm()->getConfigEntries()));
      ConfigEntry *configEntry;
      bool found = false;
      while ( ((configEntry = it.current()) != 0 ) && !found)
      {
        ++it;
        if(configEntry->getName() == Name)
        {
			configEntry->setValueFromString(Value);
			found = true;
		}
      }
      child = child.nextSibling(); 
  }

  view->getAlgorithm()->configure(view->getSizeX(), view->getSizeY(), C1, C2,*(view->getAlgorithm()->getMaxIterations()));
        
  view->configChanged();	 
  modified=false;
  return true;
}

bool FractalGeneratorDoc::saveDocument(const KURL& url, const char *format /*=0*/)
{
	FractalGeneratorView *view = pViewList->getFirst();
    QDomDocument doc( "XMLFractalSave" );
    QDomElement root = doc.createElement( "FractalV1" );
    doc.appendChild( root );


    // Create Algorithm branch
    QDomElement algorithm = doc.createElement( "Algorithm" );
    root.appendChild( algorithm );

    // Algorithm Name
    QDomElement tag = doc.createElement( "AlgorithmName" );
    algorithm.appendChild( tag );
    
    QDomText text = doc.createTextNode(view->getAlgorithm()->getSaveIdentifier());
    tag.appendChild( text );
    
    // AlgorithmC1Real
    QDomElement C = doc.createElement( "C1Real" );
    algorithm.appendChild( C );

    text = doc.createTextNode(QString().setNum(view->getAlgorithm()->getC1().real(), 'e', 64));
    C.appendChild( text );
    
    // AlgorithmC1Imag
    C = doc.createElement( "C1Imag" );
    algorithm.appendChild( C );

    text = doc.createTextNode(QString().setNum(view->getAlgorithm()->getC1().imag(), 'e', 64));
    C.appendChild( text );
    
    // AlgorithmC2Real
    C = doc.createElement( "C2Real" );
    algorithm.appendChild( C );

    text = doc.createTextNode(QString().setNum(view->getAlgorithm()->getC2().real(), 'e', 64));
    C.appendChild( text );
    
    // AlgorithmC2Imag
    C = doc.createElement( "C2Imag" );
    algorithm.appendChild( C );

    text = doc.createTextNode(QString().setNum(view->getAlgorithm()->getC2().imag(), 'e', 64));
    C.appendChild( text );


    QDomElement Optionroot = doc.createElement("Useroptions");
    algorithm.appendChild( Optionroot );
    
    QDomElement Option;
    // Fill from Config Entries
    QPtrListIterator<ConfigEntry> it(*view->getAlgorithm()->getConfigEntries());
    ConfigEntry *configEntry;
    while ( (configEntry = it.current()) != 0 ) {
         ++it;
         Option = doc.createElement( configEntry->getName() );
         Optionroot.appendChild( Option );

         text = doc.createTextNode(configEntry->getValueAsString());
         Option.appendChild( text );
    }
    

    // Create ColorScheme branch
    QDomElement colorScheme = doc.createElement( "ColorScheme" );
    root.appendChild( colorScheme );

    tag = doc.createElement( "ColorSchemeName" );
    colorScheme.appendChild( tag );

    text = doc.createTextNode( view->getColorScheme()->getSaveIdentifier() );
    tag.appendChild( text );

    
    // Resolution
    QDomElement resolution = doc.createElement( "Resolution" );
    root.appendChild( resolution );
    
    QString CurrentSize;

    CurrentSize += QString().setNum(view->getSizeX());
    CurrentSize += i18n("*");
    CurrentSize += QString().setNum(view->getSizeY());
  
    text = doc.createTextNode( CurrentSize );
    resolution.appendChild( text );

    QString Filename = url.prettyURL().mid(5) ;
    if(Filename.right(4) != ".fsf")
		Filename += ".fsf";
    QFile file( Filename);
    file.open( IO_WriteOnly );
    file.writeBlock(doc.toString(), doc.toString().length());
    modified=false;
    return true;
}

void FractalGeneratorDoc::deleteContents()
{
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////

}
