/***************************************************************************
                          fractalgeneratorview.h  -  description
                             -------------------
    begin                : Sam Mai 10 10:39:46 CEST 2003
    copyright            : (C) 2003 by Thomas Dreibholz
    email                : dreibh@exp-math.uni-essen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FRACTALGENERATORVIEW_H
#define FRACTALGENERATORVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for Qt
#include <qwidget.h>
#include <qscrollbar.h>
#include <qlayout.h>
#include <kled.h>
#include <kprinter.h>

#include "fractaldisplay.h"
#include "fractalbuffer.h"
#include "fractalalgorithminterface.h"
#include "fractalcalculationthread.h"
#include "colorschemeinterface.h"

class FractalGeneratorDoc;
class FractalGeneratorApp;

/** The FractalGeneratorView class provides the view widget for the FractalGeneratorApp instance.	
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As FractalGeneratorView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the FractalGeneratorApp class to manipulate and display
 * the document structure provided by the FractalGeneratorDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class FractalGeneratorView : public QWidget
{
  Q_OBJECT
  public:
    /** Constructor for the main view */
    FractalGeneratorView(QWidget *parent = 0, const char *name=0);
    /** Destructor for the main view */
    ~FractalGeneratorView();

    /** returns a pointer to the document connected to the view instance. Mind that this method requires a FractalGeneratorApp instance as a parent
     * widget to get to the window document pointer by calling the FractalGeneratorApp::getDocument() method.
     *
     * @see FractalGeneratorApp#getDocument
     */
    FractalGeneratorDoc *getDocument() const;
    inline FractalDisplay *getDisplay() const {return Display;}
    inline FractalAlgorithmInterface *getAlgorithm() const {return Algorithm;}
    inline ColorSchemeInterface *getColorScheme() const {return ColorScheme;}
    inline int getSizeX() {return SizeX;}
    inline int getSizeY() {return SizeY;}

    /** contains the implementation for printing functionality */
    void print(KPrinter *printer);
 
    void configChanged();

    void changeSize(int X, int Y);

    void changeAlgorithm(int index);

    void changeColorScheme(int index);

    void resetZoom();

    void zoomBack();

    bool isZoomBackPossible();

    void changeC1C2(std::complex<double> NewC1, std::complex<double> NewC2);


  public slots:
  void xScrollBarChange(int value);
  void yScrollBarChange(int value);
  void offsetUpdate(int newOffsetX, int newOffsetY);
  void selectionUpdate(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2);
  void zoomIn();

  protected:
  void resizeEvent(QResizeEvent* resizeEvent);
  bool eventFilter(QObject* object, QEvent* event);

  FractalGeneratorApp *theApp;
 
  private:
  void updateScrollBars();
  void updateView();
  void startCalculation();
  void stopCalculation();

  QScrollBar*     XScrollBar;
  QScrollBar*     YScrollBar;
  KLed*           ControlLED;
  FractalDisplay* Display;
  FractalBuffer*  Buffer;
    
  FractalCalculationThread*  Thread;
  FractalAlgorithmInterface* Algorithm;
  ColorSchemeInterface*      ColorScheme;
                                                   
  unsigned int               ProgStep;
  std::complex<double>       C1;
  std::complex<double>       C2;
  std::complex<double>       SelectionC1;
  std::complex<double>       SelectionC2;
  bool                       Selection;
  int                        SizeX;
  int                        SizeY;

  std::list<std::pair<std::complex<double>, std::complex<double> > > zoomList;
};

#endif // FRACTALGENERATORVIEW_H
