/***************************************************************************
                          optionsdialog.cpp  -  description
                             -------------------
    begin                : Mon Sep 1 2003
    copyright            : (C) 2003 by Thomas Dreibholz
    email                : dreibh@exp-math.uni-essen.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 

 
#include "optionsdialog.h"
#include "mylistviewitem.h"
#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <klocale.h>
#include <kbuttonbox.h>
#include <qptrlist.h>
#include <qtable.h>



OptionsDialog::OptionsDialog(QWidget *parent, const char *name, QPtrList<ConfigEntry> *configEntries) : QDialog(parent, name, true), m_ConfigEntries(configEntries)
{
   QVBoxLayout *top = new QVBoxLayout(this);
   top->setSpacing(10);
   top->setMargin(10);

   QLabel *lbl = new QLabel(i18n("Modify Options:"), this);
   top->addWidget(lbl);

   QFrame *frame = new QFrame(this);
   frame->setFrameStyle(QFrame::WinPanel|QFrame::Raised);
   top->addWidget(frame);
   QVBoxLayout *vbox = new QVBoxLayout(frame);
   vbox->setMargin(10);
   vbox->setSpacing(10);

    // Create the listview
   m_ListView = new QListView(frame);

   m_ListView->setColumnAlignment(0, AlignCenter);
   m_ListView->addColumn(i18n("Name"));
   m_ListView->addColumn(i18n("Value"));
   m_ListView->setAllColumnsShowFocus(true);
   m_ListView->setDefaultRenameAction(QListView::Accept);
   m_ListView->setSelectionMode(QListView::Single);


   
   
    // Fill it
    QPtrListIterator<ConfigEntry> it(*configEntries);
    ConfigEntry *configEntry;
    while ( (configEntry = it.current()) != 0 ) {
        ++it;
        MyListViewItem *Item = new MyListViewItem(m_ListView, configEntry->getName(), configEntry->getValueAsString());
		Item->setRenameEnabled(1, true);			
    }
    connect(m_ListView, SIGNAL(clicked (QListViewItem *)), SLOT(slotClicked (QListViewItem *)));
    connect(m_ListView, SIGNAL(itemRenamed ( QListViewItem *, int , const QString & )), SLOT(slotItemRenamed ( QListViewItem * , int , const QString & )));
    vbox->addWidget(m_ListView);
    // OK button
    KButtonBox *bbox = new KButtonBox(this);
    bbox->addStretch();
    QPushButton *but = bbox->addButton(i18n("&OK"));
    connect(but, SIGNAL(clicked()), SLOT(slotAccept()));
    bbox->layout();
    top->addSpacing(10);
    top->addWidget(bbox);
}


OptionsDialog::~OptionsDialog()
{
}


void OptionsDialog::slotClicked (QListViewItem *item)
{
    if(item)
        item->startRename(1);
}


void OptionsDialog::slotItemRenamed ( QListViewItem * item, int col, const QString & text )
{
    QString CurrentName = item->text(0);

    QPtrListIterator<ConfigEntry> it(*m_ConfigEntries);
    ConfigEntry *configEntry;
    while ( (configEntry = it.current()) != 0 ) {
        ++it;
        if(configEntry->getName()==CurrentName)
        {
            configEntry->setValueFromString(text);
            break;
        }
    }
}


void OptionsDialog::slotAccept()
{
    if(m_ListView->isRenaming())
    {
        MyListViewItem *item = static_cast<MyListViewItem *>(m_ListView->selectedItem());
        item->renameFinished(1);   
    }
    accept();
}
